@@ -1,62 +1,105 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
+#include <dshow.h>
+#ifdef min
+#undef min
+#endif
+#ifdef max
+#undef max
+#endif
+
 #include "directshowplayerservice.h"
 
 #include "directshowaudioendpointcontrol.h"
-#include "directshowiosource.h"
 #include "directshowmetadatacontrol.h"
+#include "vmr9videowindowcontrol.h"
+#include "directshowiosource.h"
 #include "directshowplayercontrol.h"
 #include "directshowvideorenderercontrol.h"
-#include "vmr9videowindowcontrol.h"
+#include "directshowutils.h"
+#include "directshowglobal.h"
+#include "directshowaudioprobecontrol.h"
+#include "directshowvideoprobecontrol.h"
+#include "directshowsamplegrabber.h"
 
-#ifdef HAVE_EVR
+#if QT_CONFIG(evr)
 #include "directshowevrvideowindowcontrol.h"
 #endif
 
-#ifndef QT_NO_WMSDK
-#include <wmsdk.h>
-#endif
-
 #include "qmediacontent.h"
 
+#include <QtMultimedia/private/qtmultimedia-config_p.h>
+
 #include <QtCore/qcoreapplication.h>
 #include <QtCore/qdatetime.h>
+#include <QtCore/qdir.h>
 #include <QtCore/qthread.h>
 #include <QtCore/qvarlengtharray.h>
+#include <QtCore/qsize.h>
+
+#include <QtMultimedia/qaudiobuffer.h>
+#include <QtMultimedia/qvideoframe.h>
+#include <QtMultimedia/private/qmemoryvideobuffer_p.h>
+
+#if QT_CONFIG(wmsdk)
+#  include <wmsdk.h>
+#endif
+
+#ifndef Q_CC_MINGW
+#  include <comdef.h>
+#endif
+
+QT_BEGIN_NAMESPACE
 
 Q_GLOBAL_STATIC(DirectShowEventLoop, qt_directShowEventLoop)
 
+static QString comError(HRESULT hr)
+{
+#ifndef Q_CC_MINGW // MinGW 5.3 no longer has swprintf_s().
+    _com_error error(hr);
+    return QString::fromWCharArray(error.ErrorMessage());
+#else
+    Q_UNUSED(hr)
+    return QString();
+#endif
+}
 
 // QMediaPlayer uses millisecond time units, direct show uses 100 nanosecond units.
 static const int qt_directShowTimeScale = 10000;
@@ -83,6 +126,10 @@ DirectShowPlayerService::DirectShowPlayerService(QObject *parent)
     , m_videoRendererControl(0)
     , m_videoWindowControl(0)
     , m_audioEndpointControl(0)
+    , m_audioProbeControl(nullptr)
+    , m_videoProbeControl(nullptr)
+    , m_audioSampleGrabber(nullptr)
+    , m_videoSampleGrabber(nullptr)
     , m_taskThread(0)
     , m_loop(qt_directShowEventLoop())
     , m_pendingTasks(0)
@@ -142,6 +189,8 @@ DirectShowPlayerService::~DirectShowPlayerService()
     delete m_metaDataControl;
     delete m_videoRendererControl;
     delete m_videoWindowControl;
+    delete m_audioProbeControl;
+    delete m_videoProbeControl;
 
     ::CloseHandle(m_taskHandle);
 }
@@ -167,7 +216,7 @@ QMediaControl *DirectShowPlayerService::requestControl(const char *name)
         if (!m_videoRendererControl && !m_videoWindowControl) {
             IBaseFilter *filter;
 
-#ifdef HAVE_EVR
+#if QT_CONFIG(evr)
             DirectShowEvrVideoWindowControl *evrControl = new DirectShowEvrVideoWindowControl;
             if ((filter = evrControl->filter()))
                 m_videoWindowControl = evrControl;
@@ -185,6 +234,18 @@ QMediaControl *DirectShowPlayerService::requestControl(const char *name)
 
             return m_videoWindowControl;
         }
+    } else if (qstrcmp(name, QMediaAudioProbeControl_iid) == 0) {
+        if (!m_audioProbeControl)
+            m_audioProbeControl = new DirectShowAudioProbeControl();
+        m_audioProbeControl->ref();
+        updateAudioProbe();
+        return m_audioProbeControl;
+    } else if (qstrcmp(name, QMediaVideoProbeControl_iid) == 0) {
+        if (!m_videoProbeControl)
+            m_videoProbeControl = new DirectShowVideoProbeControl();
+        m_videoProbeControl->ref();
+        updateVideoProbe();
+        return m_videoProbeControl;
     }
     return 0;
 }
@@ -206,6 +267,20 @@ void DirectShowPlayerService::releaseControl(QMediaControl *control)
         delete m_videoWindowControl;
 
         m_videoWindowControl = 0;
+    } else if (control == m_audioProbeControl) {
+        if (!m_audioProbeControl->deref()) {
+            DirectShowAudioProbeControl *old = m_audioProbeControl;
+            m_audioProbeControl = nullptr;
+            updateAudioProbe();
+            delete old;
+        }
+    } else if (control == m_videoProbeControl) {
+        if (!m_videoProbeControl->deref()) {
+            DirectShowVideoProbeControl *old = m_videoProbeControl;
+            m_videoProbeControl = nullptr;
+            updateVideoProbe();
+            delete old;
+        }
     }
 }
 
@@ -300,8 +375,10 @@ void DirectShowPlayerService::doSetUrlSource(QMutexLocker *locker)
 
     if (!SUCCEEDED(hr)) {
         locker->unlock();
+        const QString urlString = m_url.isLocalFile()
+            ? QDir::toNativeSeparators(m_url.toLocalFile()) : m_url.toString();
         hr = m_graph->AddSourceFilter(
-                reinterpret_cast<const OLECHAR *>(m_url.toString().utf16()), L"Source", &source);
+                reinterpret_cast<const OLECHAR *>(urlString.utf16()), L"Source", &source);
         locker->relock();
     }
 
@@ -313,6 +390,10 @@ void DirectShowPlayerService::doSetUrlSource(QMutexLocker *locker)
             m_pendingTasks |= SetAudioOutput;
         if (m_videoOutput)
             m_pendingTasks |= SetVideoOutput;
+        if (m_audioProbeControl)
+            m_pendingTasks |= SetAudioProbe;
+        if (m_videoProbeControl)
+            m_pendingTasks |= SetVideoProbe;
 
         if (m_rate != 1.0)
             m_pendingTasks |= SetRate;
@@ -329,17 +410,11 @@ void DirectShowPlayerService::doSetUrlSource(QMutexLocker *locker)
             m_error = QMediaPlayer::FormatError;
             m_errorString = QString();
             break;
-        case E_FAIL:
-        case E_OUTOFMEMORY:
-        case VFW_E_CANNOT_LOAD_SOURCE_FILTER:
-        case VFW_E_NOT_FOUND:
-            m_error = QMediaPlayer::ResourceError;
-            m_errorString = QString();
-            break;
         default:
             m_error = QMediaPlayer::ResourceError;
             m_errorString = QString();
-            qWarning("DirectShowPlayerService::doSetUrlSource: Unresolved error code %x", uint(hr));
+            qWarning("DirectShowPlayerService::doSetUrlSource: Unresolved error code 0x%x (%s)",
+                     uint(hr), qPrintable(comError(hr)));
             break;
         }
 
@@ -353,7 +428,8 @@ void DirectShowPlayerService::doSetStreamSource(QMutexLocker *locker)
     DirectShowIOSource *source = new DirectShowIOSource(m_loop);
     source->setDevice(m_stream);
 
-    if (SUCCEEDED(m_graph->AddFilter(source, L"Source"))) {
+    const HRESULT hr = m_graph->AddFilter(source, L"Source");
+    if (SUCCEEDED(hr)) {
         m_executedTasks = SetSource;
         m_pendingTasks |= Render;
 
@@ -374,6 +450,8 @@ void DirectShowPlayerService::doSetStreamSource(QMutexLocker *locker)
 
         m_error = QMediaPlayer::ResourceError;
         m_errorString = QString();
+        qWarning("DirectShowPlayerService::doPlay: Unresolved error code 0x%x (%s)",
+                 uint(hr), qPrintable(comError(hr)));
 
         QCoreApplication::postEvent(this, new QEvent(QEvent::Type(Error)));
     }
@@ -401,6 +479,18 @@ void DirectShowPlayerService::doRender(QMutexLocker *locker)
         m_executedTasks |= SetVideoOutput;
     }
 
+    if (m_pendingTasks & SetAudioProbe) {
+        doSetAudioProbe(locker);
+        m_pendingTasks ^= SetAudioProbe;
+        m_executedTasks |= SetAudioProbe;
+    }
+
+    if (m_pendingTasks & SetVideoProbe) {
+        doSetVideoProbe(locker);
+        m_pendingTasks ^= SetVideoProbe;
+        m_executedTasks |= SetVideoProbe;
+    }
+
     IFilterGraph2 *graph = m_graph;
     graph->AddRef();
 
@@ -489,8 +579,8 @@ void DirectShowPlayerService::doRender(QMutexLocker *locker)
                 default:
                     m_error = QMediaPlayer::ResourceError;
                     m_errorString = QString();
-                    qWarning("DirectShowPlayerService::doRender: Unresolved error code %x",
-                             uint(renderHr));
+                    qWarning("DirectShowPlayerService::doRender: Unresolved error code 0x%x (%s)",
+                             uint(renderHr), qPrintable(comError(renderHr)));
                 }
             }
 
@@ -499,8 +589,6 @@ void DirectShowPlayerService::doRender(QMutexLocker *locker)
 
         m_executedTasks |= Render;
     }
-
-    m_loop->wake();
 }
 
 void DirectShowPlayerService::doFinalizeLoad(QMutexLocker *locker)
@@ -570,6 +658,9 @@ void DirectShowPlayerService::doReleaseGraph(QMutexLocker *locker)
         control->Release();
     }
 
+    doReleaseAudioProbe(locker);
+    doReleaseVideoProbe(locker);
+
     if (m_source) {
         m_source->Release();
         m_source = 0;
@@ -583,6 +674,139 @@ void DirectShowPlayerService::doReleaseGraph(QMutexLocker *locker)
     m_loop->wake();
 }
 
+void DirectShowPlayerService::doSetVideoProbe(QMutexLocker *locker)
+{
+    Q_UNUSED(locker);
+
+    if (!m_graph) {
+        qCWarning(qtDirectShowPlugin, "Attempting to set a video probe without a valid graph!");
+        return;
+    }
+
+    // Create the sample grabber, if necessary.
+    if (!m_videoSampleGrabber) {
+        m_videoSampleGrabber = new DirectShowSampleGrabber;
+        connect(m_videoSampleGrabber, &DirectShowSampleGrabber::bufferAvailable, this, &DirectShowPlayerService::onVideoBufferAvailable);
+    }
+
+    if (FAILED(m_graph->AddFilter(m_videoSampleGrabber->filter(), L"Video Sample Grabber"))) {
+        qCWarning(qtDirectShowPlugin, "Failed to add the video sample grabber into the graph!");
+        return;
+    }
+
+    // TODO: Make util function for getting this, so it's easy to keep it in sync.
+    static const GUID subtypes[] = { MEDIASUBTYPE_ARGB32,
+                                     MEDIASUBTYPE_RGB32,
+                                     MEDIASUBTYPE_RGB24,
+                                     MEDIASUBTYPE_RGB565,
+                                     MEDIASUBTYPE_RGB555,
+                                     MEDIASUBTYPE_AYUV,
+                                     MEDIASUBTYPE_I420,
+                                     MEDIASUBTYPE_IYUV,
+                                     MEDIASUBTYPE_YV12,
+                                     MEDIASUBTYPE_UYVY,
+                                     MEDIASUBTYPE_YUYV,
+                                     MEDIASUBTYPE_YUY2,
+                                     MEDIASUBTYPE_NV12,
+                                     MEDIASUBTYPE_MJPG,
+                                     MEDIASUBTYPE_IMC1,
+                                     MEDIASUBTYPE_IMC2,
+                                     MEDIASUBTYPE_IMC3,
+                                     MEDIASUBTYPE_IMC4 };
+
+    // Negotiate the subtype
+    DirectShowMediaType mediaType(AM_MEDIA_TYPE { MEDIATYPE_Video });
+    const int items = (sizeof subtypes / sizeof(GUID));
+    bool connected = false;
+    for (int i = 0; i != items; ++i) {
+        mediaType->subtype = subtypes[i];
+        m_videoSampleGrabber->setMediaType(&mediaType);
+        if (SUCCEEDED(DirectShowUtils::connectFilters(m_graph, m_source, m_videoSampleGrabber->filter(), true)))
+            connected = true;
+            break;
+    }
+
+    if (!connected) {
+        qCWarning(qtDirectShowPlugin, "Unable to connect the video probe!");
+        return;
+    }
+
+    m_videoSampleGrabber->start(DirectShowSampleGrabber::CallbackMethod::BufferCB);
+}
+
+void DirectShowPlayerService::doSetAudioProbe(QMutexLocker *locker)
+{
+    Q_UNUSED(locker);
+
+    if (!m_graph) {
+        qCWarning(qtDirectShowPlugin, "Attempting to set an audio probe without a valid graph!");
+        return;
+    }
+
+    // Create the sample grabber, if necessary.
+    if (!m_audioSampleGrabber) {
+        m_audioSampleGrabber = new DirectShowSampleGrabber;
+        connect(m_audioSampleGrabber, &DirectShowSampleGrabber::bufferAvailable, this, &DirectShowPlayerService::onAudioBufferAvailable);
+    }
+
+    static const AM_MEDIA_TYPE mediaType { MEDIATYPE_Audio, MEDIASUBTYPE_PCM };
+    m_audioSampleGrabber->setMediaType(&mediaType);
+
+    if (FAILED(m_graph->AddFilter(m_audioSampleGrabber->filter(), L"Audio Sample Grabber"))) {
+        qCWarning(qtDirectShowPlugin, "Failed to add the audio sample grabber into the graph!");
+        return;
+    }
+
+    if (FAILED(DirectShowUtils::connectFilters(m_graph, m_source, m_audioSampleGrabber->filter(), true))) {
+        qCWarning(qtDirectShowPlugin, "Failed to connect the audio sample grabber");
+        return;
+    }
+
+    m_audioSampleGrabber->start(DirectShowSampleGrabber::CallbackMethod::BufferCB);
+}
+
+void DirectShowPlayerService::doReleaseVideoProbe(QMutexLocker *locker)
+{
+    Q_UNUSED(locker);
+
+    if (!m_graph)
+        return;
+
+    if (!m_videoSampleGrabber)
+        return;
+
+    m_videoSampleGrabber->stop();
+    HRESULT hr = m_graph->RemoveFilter(m_videoSampleGrabber->filter());
+    if (FAILED(hr)) {
+        qCWarning(qtDirectShowPlugin, "Failed to remove the video sample grabber!");
+        return;
+    }
+
+    m_videoSampleGrabber->deleteLater();
+    m_videoSampleGrabber = nullptr;
+}
+
+void DirectShowPlayerService::doReleaseAudioProbe(QMutexLocker *locker)
+{
+    Q_UNUSED(locker);
+
+    if (!m_graph)
+        return;
+
+    if (!m_audioSampleGrabber)
+        return;
+
+    m_audioSampleGrabber->stop();
+    HRESULT hr = m_graph->RemoveFilter(m_audioSampleGrabber->filter());
+    if (FAILED(hr)) {
+        qCWarning(qtDirectShowPlugin, "Failed to remove the audio sample grabber!");
+        return;
+    }
+
+    m_audioSampleGrabber->deleteLater();
+    m_audioSampleGrabber = nullptr;
+}
+
 int DirectShowPlayerService::findStreamTypes(IBaseFilter *source) const
 {
     QVarLengthArray<IBaseFilter *, 16> filters;
@@ -600,13 +824,13 @@ int DirectShowPlayerService::findStreamTypes(IBaseFilter *source) const
             for (IPin *pin = 0; pins->Next(1, &pin, 0) == S_OK; pin->Release()) {
                 PIN_DIRECTION direction;
                 if (pin->QueryDirection(&direction) == S_OK && direction == PINDIR_OUTPUT) {
-                    AM_MEDIA_TYPE connectionType;
+                    DirectShowMediaType connectionType;
                     if (SUCCEEDED(pin->ConnectionMediaType(&connectionType))) {
                         IPin *peer = 0;
 
-                        if (connectionType.majortype == MEDIATYPE_Audio) {
+                        if (connectionType->majortype == MEDIATYPE_Audio) {
                             streamTypes |= AudioStream;
-                        } else if (connectionType.majortype == MEDIATYPE_Video) {
+                        } else if (connectionType->majortype == MEDIATYPE_Video) {
                             streamTypes |= VideoStream;
                         } else if (SUCCEEDED(pin->ConnectedTo(&peer))) {
                             PIN_INFO peerInfo;
@@ -701,7 +925,8 @@ void DirectShowPlayerService::doPlay(QMutexLocker *locker)
         } else {
             m_error = QMediaPlayer::ResourceError;
             m_errorString = QString();
-            qWarning("DirectShowPlayerService::doPlay: Unresolved error code %x", uint(hr));
+            qWarning("DirectShowPlayerService::doPlay: Unresolved error code 0x%x (%s)",
+                     uint(hr), qPrintable(comError(hr)));
 
             QCoreApplication::postEvent(this, new QEvent(QEvent::Type(Error)));
         }
@@ -760,7 +985,8 @@ void DirectShowPlayerService::doPause(QMutexLocker *locker)
         } else {
             m_error = QMediaPlayer::ResourceError;
             m_errorString = QString();
-            qWarning("DirectShowPlayerService::doPause: Unresolved error code %x", uint(hr));
+            qWarning("DirectShowPlayerService::doPause: Unresolved error code 0x%x (%s)",
+                     uint(hr), qPrintable(comError(hr)));
 
             QCoreApplication::postEvent(this, new QEvent(QEvent::Type(Error)));
         }
@@ -951,7 +1177,7 @@ void DirectShowPlayerService::doSeek(QMutexLocker *locker)
 
 int DirectShowPlayerService::bufferStatus() const
 {
-#ifndef QT_NO_WMSDK
+#if QT_CONFIG(wmsdk)
     QMutexLocker locker(const_cast<QMutex *>(&m_mutex));
 
     if (IWMReaderAdvanced2 *reader = com_cast<IWMReaderAdvanced2>(
@@ -1088,6 +1314,40 @@ void DirectShowPlayerService::setVideoOutput(IBaseFilter *filter)
     }
 }
 
+void DirectShowPlayerService::updateAudioProbe()
+{
+    QMutexLocker locker(&m_mutex);
+
+    // Set/Activate the audio probe.
+    if (m_graph) {
+        // If we don't have a audio probe, then stop and release the audio sample grabber
+        if (!m_audioProbeControl && (m_executedTasks & SetAudioProbe)) {
+            m_pendingTasks |= ReleaseAudioProbe;
+            ::SetEvent(m_taskHandle);
+            m_loop->wait(&m_mutex);
+        } else if (m_audioProbeControl) {
+            m_pendingTasks |= SetAudioProbe;
+        }
+    }
+}
+
+void DirectShowPlayerService::updateVideoProbe()
+{
+    QMutexLocker locker(&m_mutex);
+
+    // Set/Activate the video probe.
+    if (m_graph) {
+        // If we don't have a video probe, then stop and release the video sample grabber
+        if (!m_videoProbeControl && (m_executedTasks & SetVideoProbe)) {
+            m_pendingTasks |= ReleaseVideoProbe;
+            ::SetEvent(m_taskHandle);
+            m_loop->wait(&m_mutex);
+        } else if (m_videoProbeControl){
+            m_pendingTasks |= SetVideoProbe;
+        }
+    }
+}
+
 void DirectShowPlayerService::doReleaseVideoOutput(QMutexLocker *locker)
 {
     Q_UNUSED(locker)
@@ -1184,6 +1444,97 @@ void DirectShowPlayerService::videoOutputChanged()
     setVideoOutput(m_videoRendererControl->filter());
 }
 
+void DirectShowPlayerService::onAudioBufferAvailable(double time, quint8 *buffer, long len)
+{
+    QMutexLocker locker(&m_mutex);
+    if (!m_audioProbeControl || !m_audioSampleGrabber)
+        return;
+
+    DirectShowMediaType mt(AM_MEDIA_TYPE { GUID_NULL });
+    const bool ok = m_audioSampleGrabber->getConnectedMediaType(&mt);
+    if (!ok)
+        return;
+
+    if (mt->majortype != MEDIATYPE_Audio)
+        return;
+
+    if (mt->subtype != MEDIASUBTYPE_PCM)
+        return;
+
+    const bool isWfx = ((mt->formattype == FORMAT_WaveFormatEx) && (mt->cbFormat >= sizeof(WAVEFORMATEX)));
+    WAVEFORMATEX *wfx = isWfx ? reinterpret_cast<WAVEFORMATEX *>(mt->pbFormat) : nullptr;
+
+    if (!wfx)
+        return;
+
+    if (wfx->wFormatTag != WAVE_FORMAT_PCM && wfx->wFormatTag != WAVE_FORMAT_EXTENSIBLE)
+        return;
+
+    if ((wfx->wFormatTag == WAVE_FORMAT_EXTENSIBLE) && (wfx->cbSize >= sizeof(WAVEFORMATEXTENSIBLE))) {
+        WAVEFORMATEXTENSIBLE *wfxe = reinterpret_cast<WAVEFORMATEXTENSIBLE *>(wfx);
+        if (wfxe->SubFormat != KSDATAFORMAT_SUBTYPE_PCM)
+            return;
+    }
+
+    QAudioFormat format;
+    format.setSampleRate(wfx->nSamplesPerSec);
+    format.setChannelCount(wfx->nChannels);
+    format.setSampleSize(wfx->wBitsPerSample);
+    format.setCodec("audio/pcm");
+    format.setByteOrder(QAudioFormat::LittleEndian);
+    if (format.sampleSize() == 8)
+        format.setSampleType(QAudioFormat::UnSignedInt);
+    else
+        format.setSampleType(QAudioFormat::SignedInt);
+
+    const quint64 startTime = quint64(time * 1000.);
+    QAudioBuffer audioBuffer(QByteArray(reinterpret_cast<const char *>(buffer), len),
+                             format,
+                             startTime);
+
+    Q_EMIT m_audioProbeControl->audioBufferProbed(audioBuffer);
+}
+
+void DirectShowPlayerService::onVideoBufferAvailable(double time, quint8 *buffer, long len)
+{
+    Q_UNUSED(time);
+    Q_UNUSED(buffer);
+    Q_UNUSED(len);
+
+    QMutexLocker locker(&m_mutex);
+    if (!m_videoProbeControl || !m_videoSampleGrabber)
+        return;
+
+    DirectShowMediaType mt(AM_MEDIA_TYPE { GUID_NULL });
+    const bool ok = m_videoSampleGrabber->getConnectedMediaType(&mt);
+    if (!ok)
+        return;
+
+    if (mt->majortype != MEDIATYPE_Video)
+        return;
+
+    QVideoFrame::PixelFormat format = DirectShowMediaType::pixelFormatFromType(&mt);
+    if (format == QVideoFrame::Format_Invalid) {
+        qCWarning(qtDirectShowPlugin, "Invalid format, stopping video probes!");
+        m_videoSampleGrabber->stop();
+        return;
+    }
+
+    const QVideoSurfaceFormat &videoFormat = DirectShowMediaType::videoFormatFromType(&mt);
+    if (!videoFormat.isValid())
+        return;
+
+    const QSize &size = videoFormat.frameSize();
+
+    const int bytesPerLine = DirectShowMediaType::bytesPerLine(videoFormat);
+    QByteArray data(reinterpret_cast<const char *>(buffer), len);
+    QVideoFrame frame(new QMemoryVideoBuffer(data, bytesPerLine),
+                      size,
+                      format);
+
+    Q_EMIT m_videoProbeControl->videoFrameProbed(frame);
+}
+
 void DirectShowPlayerService::graphEvent(QMutexLocker *locker)
 {
     Q_UNUSED(locker)
@@ -1372,6 +1723,16 @@ void DirectShowPlayerService::run()
             m_executingTask = ReleaseVideoOutput;
 
             doReleaseVideoOutput(&locker);
+        } else if (m_pendingTasks & ReleaseAudioProbe) {
+            m_pendingTasks ^= ReleaseAudioProbe;
+            m_executingTask = ReleaseAudioProbe;
+
+            doReleaseAudioProbe(&locker);
+        } else if (m_pendingTasks & ReleaseVideoProbe) {
+            m_pendingTasks ^= ReleaseVideoProbe;
+            m_executingTask = ReleaseVideoProbe;
+
+            doReleaseVideoProbe(&locker);
         } else if (m_pendingTasks & SetUrlSource) {
             m_pendingTasks ^= SetUrlSource;
             m_executingTask = SetUrlSource;
@@ -1382,6 +1743,16 @@ void DirectShowPlayerService::run()
             m_executingTask = SetStreamSource;
 
             doSetStreamSource(&locker);
+        } else if (m_pendingTasks & SetAudioProbe) {
+            m_pendingTasks ^= SetAudioProbe;
+            m_executingTask = SetAudioProbe;
+
+            doSetAudioProbe(&locker);
+        } else if (m_pendingTasks & SetVideoProbe) {
+            m_pendingTasks ^= SetVideoProbe;
+            m_executingTask = SetVideoProbe;
+
+            doSetVideoProbe(&locker);
         } else if (m_pendingTasks & Render) {
             m_pendingTasks ^= Render;
             m_executingTask = Render;
@@ -1423,3 +1794,5 @@ void DirectShowPlayerService::run()
         m_executingTask = 0;
     }
 }
+
+QT_END_NAMESPACE
