@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -49,17 +55,18 @@
 #include <QtCore/qurl.h>
 #include <QtCore/qwaitcondition.h>
 
+QT_BEGIN_NAMESPACE
+
 class DirectShowAudioEndpointControl;
 class DirectShowMetaDataControl;
 class DirectShowPlayerControl;
 class DirectShowVideoRendererControl;
+class DirectShowAudioProbeControl;
+class DirectShowVideoProbeControl;
+class DirectShowSampleGrabber;
 
-QT_BEGIN_NAMESPACE
 class QMediaContent;
 class QVideoWindowControl;
-QT_END_NAMESPACE
-
-QT_USE_NAMESPACE
 
 class DirectShowPlayerService : public QMediaService
 {
@@ -99,10 +106,16 @@ protected:
 private Q_SLOTS:
     void videoOutputChanged();
 
+    void onAudioBufferAvailable(double time, quint8 *buffer, long len);
+    void onVideoBufferAvailable(double time, quint8 *buffer, long len);
+
 private:
     void releaseGraph();
     void updateStatus();
 
+    void updateAudioProbe();
+    void updateVideoProbe();
+
     int findStreamTypes(IBaseFilter *source) const;
     int findStreamType(IPin *pin) const;
 
@@ -123,29 +136,40 @@ private:
     void doReleaseAudioOutput(QMutexLocker *locker);
     void doReleaseVideoOutput(QMutexLocker *locker);
     void doReleaseGraph(QMutexLocker *locker);
+    void doSetVideoProbe(QMutexLocker *locker);
+    void doSetAudioProbe(QMutexLocker *locker);
+    void doReleaseVideoProbe(QMutexLocker *locker);
+    void doReleaseAudioProbe(QMutexLocker *locker);
 
     void graphEvent(QMutexLocker *locker);
 
     enum Task
     {
-        Shutdown           = 0x0001,
-        SetUrlSource       = 0x0002,
-        SetStreamSource    = 0x0004,
+        Shutdown           = 0x00001,
+        SetUrlSource       = 0x00002,
+        SetStreamSource    = 0x00004,
         SetSource          = SetUrlSource | SetStreamSource,
-        SetAudioOutput     = 0x0008,
-        SetVideoOutput     = 0x0010,
+        SetAudioOutput     = 0x00008,
+        SetVideoOutput     = 0x00010,
         SetOutputs         = SetAudioOutput | SetVideoOutput,
-        Render             = 0x0020,
-        FinalizeLoad       = 0x0040,
-        SetRate            = 0x0080,
-        Seek               = 0x0100,
-        Play               = 0x0200,
-        Pause              = 0x0400,
-        Stop               = 0x0800,
-        ReleaseGraph       = 0x1000,
-        ReleaseAudioOutput = 0x2000,
-        ReleaseVideoOutput = 0x4000,
-        ReleaseFilters     = ReleaseGraph | ReleaseAudioOutput | ReleaseVideoOutput
+        SetAudioProbe      = 0x00020,
+        SetVideoProbe      = 0x00040,
+        SetProbes          = SetAudioProbe | SetVideoProbe,
+        Render             = 0x00080,
+        FinalizeLoad       = 0x00100,
+        SetRate            = 0x00200,
+        Seek               = 0x00400,
+        Play               = 0x00800,
+        Pause              = 0x01000,
+        Stop               = 0x02000,
+        ReleaseGraph       = 0x04000,
+        ReleaseAudioOutput = 0x08000,
+        ReleaseVideoOutput = 0x10000,
+        ReleaseAudioProbe  = 0x20000,
+        ReleaseVideoProbe  = 0x40000,
+        ReleaseFilters     = ReleaseGraph | ReleaseAudioOutput
+                             | ReleaseVideoOutput | ReleaseAudioProbe
+                             | ReleaseVideoProbe
     };
 
     enum Event
@@ -170,20 +194,21 @@ private:
     };
 
     DirectShowPlayerControl *m_playerControl;
-#ifndef Q_OS_WINCE
     DirectShowMetaDataControl *m_metaDataControl;
-#endif
     DirectShowVideoRendererControl *m_videoRendererControl;
-#ifndef Q_OS_WINCE
     QVideoWindowControl *m_videoWindowControl;
     DirectShowAudioEndpointControl *m_audioEndpointControl;
-#endif
+    DirectShowAudioProbeControl *m_audioProbeControl;
+    DirectShowVideoProbeControl *m_videoProbeControl;
+    DirectShowSampleGrabber *m_audioSampleGrabber;
+    DirectShowSampleGrabber *m_videoSampleGrabber;
 
     QThread *m_taskThread;
     DirectShowEventLoop *m_loop;
     int m_pendingTasks;
     int m_executingTask;
     int m_executedTasks;
+    int m_streamTypes;
     HANDLE m_taskHandle;
     HANDLE m_eventHandle;
     GraphStatus m_graphStatus;
@@ -193,23 +218,23 @@ private:
     IBaseFilter *m_source;
     IBaseFilter *m_audioOutput;
     IBaseFilter *m_videoOutput;
-    int m_streamTypes;
     qreal m_rate;
     qint64 m_position;
     qint64 m_seekPosition;
     qint64 m_duration;
-    bool m_buffering;
-    bool m_seekable;
-    bool m_atEnd;
-    bool m_dontCacheNextSeekResult;
     QMediaTimeRange m_playbackRange;
     QUrl m_url;
     QMediaResourceList m_resources;
     QString m_errorString;
     QMutex m_mutex;
+    bool m_buffering;
+    bool m_seekable;
+    bool m_atEnd;
+    bool m_dontCacheNextSeekResult;
 
     friend class DirectShowPlayerServiceThread;
 };
 
+QT_END_NAMESPACE
 
 #endif
